# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.email_routing import dns_get_params, dns_edit_params, dns_create_params
from ...types.email_routing.settings import Settings
from ...types.email_routing.dns_record import DNSRecord
from ...types.email_routing.dns_get_response import DNSGetResponse

__all__ = ["DNSResource", "AsyncDNSResource"]


class DNSResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DNSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DNSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DNSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DNSResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Enable you Email Routing zone.

        Add and lock the necessary MX and SPF records.

        Args:
          zone_id: Identifier.

          name: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            body=maybe_transform({"name": name}, dns_create_params.DNSCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DNSRecord]:
        """Disable your Email Routing zone.

        Also removes additional MX records previously
        required for Email Routing to work.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            page=SyncSinglePage[DNSRecord],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DNSRecord,
            method="delete",
        )

    def edit(
        self,
        *,
        zone_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """
        Unlock MX Records previously locked by Email Routing.

        Args:
          zone_id: Identifier.

          name: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            body=maybe_transform({"name": name}, dns_edit_params.DNSEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    def get(
        self,
        *,
        zone_id: str,
        subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DNSGetResponse:
        """
        Show the DNS records needed to configure your Email Routing zone.

        Args:
          zone_id: Identifier.

          subdomain: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            DNSGetResponse,
            self._get(
                path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform({"subdomain": subdomain}, dns_get_params.DNSGetParams),
                ),
                cast_to=cast(Any, DNSGetResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncDNSResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDNSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDNSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDNSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDNSResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Enable you Email Routing zone.

        Add and lock the necessary MX and SPF records.

        Args:
          zone_id: Identifier.

          name: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            body=await async_maybe_transform({"name": name}, dns_create_params.DNSCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DNSRecord, AsyncSinglePage[DNSRecord]]:
        """Disable your Email Routing zone.

        Also removes additional MX records previously
        required for Email Routing to work.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            page=AsyncSinglePage[DNSRecord],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DNSRecord,
            method="delete",
        )

    async def edit(
        self,
        *,
        zone_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """
        Unlock MX Records previously locked by Email Routing.

        Args:
          zone_id: Identifier.

          name: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
            body=await async_maybe_transform({"name": name}, dns_edit_params.DNSEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DNSGetResponse:
        """
        Show the DNS records needed to configure your Email Routing zone.

        Args:
          zone_id: Identifier.

          subdomain: Domain of your zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            DNSGetResponse,
            await self._get(
                path_template("/zones/{zone_id}/email/routing/dns", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform({"subdomain": subdomain}, dns_get_params.DNSGetParams),
                ),
                cast_to=cast(Any, DNSGetResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class DNSResourceWithRawResponse:
    def __init__(self, dns: DNSResource) -> None:
        self._dns = dns

        self.create = to_raw_response_wrapper(
            dns.create,
        )
        self.delete = to_raw_response_wrapper(
            dns.delete,
        )
        self.edit = to_raw_response_wrapper(
            dns.edit,
        )
        self.get = to_raw_response_wrapper(
            dns.get,
        )


class AsyncDNSResourceWithRawResponse:
    def __init__(self, dns: AsyncDNSResource) -> None:
        self._dns = dns

        self.create = async_to_raw_response_wrapper(
            dns.create,
        )
        self.delete = async_to_raw_response_wrapper(
            dns.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            dns.edit,
        )
        self.get = async_to_raw_response_wrapper(
            dns.get,
        )


class DNSResourceWithStreamingResponse:
    def __init__(self, dns: DNSResource) -> None:
        self._dns = dns

        self.create = to_streamed_response_wrapper(
            dns.create,
        )
        self.delete = to_streamed_response_wrapper(
            dns.delete,
        )
        self.edit = to_streamed_response_wrapper(
            dns.edit,
        )
        self.get = to_streamed_response_wrapper(
            dns.get,
        )


class AsyncDNSResourceWithStreamingResponse:
    def __init__(self, dns: AsyncDNSResource) -> None:
        self._dns = dns

        self.create = async_to_streamed_response_wrapper(
            dns.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            dns.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            dns.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            dns.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WorkerListParams"]


class WorkerListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    order: Literal["asc", "desc"]
    """Sort direction."""

    order_by: Literal["deployed_on", "updated_on", "created_on", "name"]
    """Property to sort results by."""

    page: int
    """Current page."""

    per_page: int
    """Items per-page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.email_security import submission_list_params
from ...types.email_security.submission_list_response import SubmissionListResponse

__all__ = ["SubmissionsResource", "AsyncSubmissionsResource"]


class SubmissionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubmissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubmissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubmissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubmissionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        end: Union[str, datetime] | Omit = omit,
        escalated_from_user: bool | Omit = omit,
        original_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        outcome_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        query: Optional[str] | Omit = omit,
        requested_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        status: str | Omit = omit,
        submission_id: str | Omit = omit,
        type: Literal["TEAM", "USER"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[SubmissionListResponse]:
        """Returns information for submissions made to reclassify emails.

        Shows the status,
        outcome, and disposition changes for reclassification requests made by users or
        the security team. Useful for tracking false positive/negative reports.

        Args:
          account_id: Identifier.

          end: The end of the search date range. Defaults to `now`.

          escalated_from_user: When true, return only submissions that were escalated by an end user (vs. by
              the security team). When false, return only submissions that were not escalated
              by an end user. When omitted, no filter is applied.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          start: The beginning of the search date range. Defaults to `now - 30 days`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/submissions", account_id=account_id),
            page=SyncV4PagePaginationArray[SubmissionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end": end,
                        "escalated_from_user": escalated_from_user,
                        "original_disposition": original_disposition,
                        "outcome_disposition": outcome_disposition,
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "requested_disposition": requested_disposition,
                        "start": start,
                        "status": status,
                        "submission_id": submission_id,
                        "type": type,
                    },
                    submission_list_params.SubmissionListParams,
                ),
            ),
            model=SubmissionListResponse,
        )


class AsyncSubmissionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubmissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubmissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubmissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubmissionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        end: Union[str, datetime] | Omit = omit,
        escalated_from_user: bool | Omit = omit,
        original_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        outcome_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        query: Optional[str] | Omit = omit,
        requested_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        status: str | Omit = omit,
        submission_id: str | Omit = omit,
        type: Literal["TEAM", "USER"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SubmissionListResponse, AsyncV4PagePaginationArray[SubmissionListResponse]]:
        """Returns information for submissions made to reclassify emails.

        Shows the status,
        outcome, and disposition changes for reclassification requests made by users or
        the security team. Useful for tracking false positive/negative reports.

        Args:
          account_id: Identifier.

          end: The end of the search date range. Defaults to `now`.

          escalated_from_user: When true, return only submissions that were escalated by an end user (vs. by
              the security team). When false, return only submissions that were not escalated
              by an end user. When omitted, no filter is applied.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          start: The beginning of the search date range. Defaults to `now - 30 days`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/submissions", account_id=account_id),
            page=AsyncV4PagePaginationArray[SubmissionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end": end,
                        "escalated_from_user": escalated_from_user,
                        "original_disposition": original_disposition,
                        "outcome_disposition": outcome_disposition,
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "requested_disposition": requested_disposition,
                        "start": start,
                        "status": status,
                        "submission_id": submission_id,
                        "type": type,
                    },
                    submission_list_params.SubmissionListParams,
                ),
            ),
            model=SubmissionListResponse,
        )


class SubmissionsResourceWithRawResponse:
    def __init__(self, submissions: SubmissionsResource) -> None:
        self._submissions = submissions

        self.list = to_raw_response_wrapper(
            submissions.list,
        )


class AsyncSubmissionsResourceWithRawResponse:
    def __init__(self, submissions: AsyncSubmissionsResource) -> None:
        self._submissions = submissions

        self.list = async_to_raw_response_wrapper(
            submissions.list,
        )


class SubmissionsResourceWithStreamingResponse:
    def __init__(self, submissions: SubmissionsResource) -> None:
        self._submissions = submissions

        self.list = to_streamed_response_wrapper(
            submissions.list,
        )


class AsyncSubmissionsResourceWithStreamingResponse:
    def __init__(self, submissions: AsyncSubmissionsResource) -> None:
        self._submissions = submissions

        self.list = async_to_streamed_response_wrapper(
            submissions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .pcap_filter import PCAPFilter

__all__ = ["PCAP"]


class PCAP(BaseModel):
    id: Optional[str] = None
    """The ID for the packet capture."""

    filter_v1: Optional[PCAPFilter] = None
    """The packet capture filter. When this field is empty, all packets are captured."""

    offset_time: Optional[datetime] = None
    """The RFC 3339 offset timestamp from which to query backwards for packets.

    Must be within the last 24h. When this field is empty, defaults to time of
    request.
    """

    status: Optional[
        Literal[
            "unknown", "success", "pending", "running", "conversion_pending", "conversion_running", "complete", "failed"
        ]
    ] = None
    """The status of the packet capture request."""

    submitted: Optional[str] = None
    """The RFC 3339 timestamp when the packet capture was created."""

    system: Optional[Literal["magic-transit"]] = None
    """The system used to collect packet captures."""

    time_limit: Optional[float] = None
    """The packet capture duration in seconds."""

    type: Optional[Literal["simple", "full"]] = None
    """The type of packet capture.

    `Simple` captures sampled packets, and `full` captures entire payloads and
    non-sampled packets.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["SessionGetSessionSummaryResponse", "Data"]


class Data(BaseModel):
    session_id: str = FieldInfo(alias="sessionId")

    summary_download_url: str = FieldInfo(alias="summaryDownloadUrl")
    """URL where the summary of transcripts can be downloaded"""

    summary_download_url_expiry: str = FieldInfo(alias="summaryDownloadUrlExpiry")
    """Time of Expiry before when you need to download the csv file."""


class SessionGetSessionSummaryResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.workers.scripts import setting_edit_params
from ....types.workers.script_setting import ScriptSetting
from ....types.workers.scripts.consumer_script_param import ConsumerScriptParam

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def edit(
        self,
        script_name: str,
        *,
        account_id: str,
        logpush: bool | Omit = omit,
        observability: Optional[setting_edit_params.Observability] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        tail_consumers: Optional[Iterable[ConsumerScriptParam]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScriptSetting:
        """
        Patch script-level settings when using
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions).
        Including but not limited to Logpush and Tail Consumers.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          logpush: Whether Logpush is turned on for the Worker.

          observability: Observability settings for the Worker.

          tags: Tags associated with the Worker.

          tail_consumers: List of Workers that will consume logs from the attached Worker.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/script-settings",
                account_id=account_id,
                script_name=script_name,
            ),
            body=maybe_transform(
                {
                    "logpush": logpush,
                    "observability": observability,
                    "tags": tags,
                    "tail_consumers": tail_consumers,
                },
                setting_edit_params.SettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ScriptSetting]._unwrapper,
            ),
            cast_to=cast(Type[ScriptSetting], ResultWrapper[ScriptSetting]),
        )

    def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScriptSetting:
        """
        Get script-level settings when using
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions).
        Includes Logpush and Tail Consumers.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/script-settings",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ScriptSetting]._unwrapper,
            ),
            cast_to=cast(Type[ScriptSetting], ResultWrapper[ScriptSetting]),
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def edit(
        self,
        script_name: str,
        *,
        account_id: str,
        logpush: bool | Omit = omit,
        observability: Optional[setting_edit_params.Observability] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        tail_consumers: Optional[Iterable[ConsumerScriptParam]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScriptSetting:
        """
        Patch script-level settings when using
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions).
        Including but not limited to Logpush and Tail Consumers.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          logpush: Whether Logpush is turned on for the Worker.

          observability: Observability settings for the Worker.

          tags: Tags associated with the Worker.

          tail_consumers: List of Workers that will consume logs from the attached Worker.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/script-settings",
                account_id=account_id,
                script_name=script_name,
            ),
            body=await async_maybe_transform(
                {
                    "logpush": logpush,
                    "observability": observability,
                    "tags": tags,
                    "tail_consumers": tail_consumers,
                },
                setting_edit_params.SettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ScriptSetting]._unwrapper,
            ),
            cast_to=cast(Type[ScriptSetting], ResultWrapper[ScriptSetting]),
        )

    async def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScriptSetting:
        """
        Get script-level settings when using
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions).
        Includes Logpush and Tail Consumers.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/script-settings",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ScriptSetting]._unwrapper,
            ),
            cast_to=cast(Type[ScriptSetting], ResultWrapper[ScriptSetting]),
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.edit = to_raw_response_wrapper(
            settings.edit,
        )
        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.edit = async_to_raw_response_wrapper(
            settings.edit,
        )
        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.edit = to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.edit = async_to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["ASNAsSetResponse", "AsSet"]


class AsSet(BaseModel):
    as_members_count: int
    """The number of AS members in the AS-SET"""

    as_set_members_count: int
    """The number of AS-SET members in the AS-SET"""

    as_set_upstreams_count: int
    """The number of recursive upstream AS-SETs"""

    asn_cone_size: int
    """The number of unique ASNs in the AS-SETs recursive downstream"""

    irr_sources: List[str]
    """The IRR sources of the AS-SET"""

    name: str
    """The name of the AS-SET"""

    hierarchical_asn: Optional[int] = None
    """The AS number following hierarchical AS-SET name"""

    inferred_asn: Optional[int] = None
    """The inferred AS number of the AS-SET"""

    peeringdb_asn: Optional[int] = None
    """The AS number matching PeeringDB record"""


class ASNAsSetResponse(BaseModel):
    as_sets: List[AsSet]

    paths: List[List[str]]
    """Paths from the AS-SET that include the given AS to its upstreams recursively"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._files import read_file_content, async_read_file_content
from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    BinaryTypes,
    FileContent,
    AsyncBinaryTypes,
    omit,
    not_given,
)
from ...._utils import is_given, path_template, maybe_transform, strip_not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncCursorPagination, AsyncCursorPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.r2.buckets import object_list_params
from ....types.r2.buckets.object_list_response import ObjectListResponse
from ....types.r2.buckets.object_delete_response import ObjectDeleteResponse
from ....types.r2.buckets.object_upload_response import ObjectUploadResponse

__all__ = ["ObjectsResource", "AsyncObjectsResource"]


class ObjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ObjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ObjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ObjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ObjectsResourceWithStreamingResponse(self)

    def list(
        self,
        bucket_name: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        delimiter: str | Omit = omit,
        per_page: int | Omit = omit,
        prefix: str | Omit = omit,
        start_after: str | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[ObjectListResponse]:
        """Lists objects in an R2 bucket.

        Returns object metadata including key, size,
        etag, last modified date, HTTP metadata, and custom metadata.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          cursor: Pagination cursor received from a previous List Objects call. Used to retrieve
              the next page of results.

          delimiter: A single character used to group keys. All keys that contain the delimiter
              between the prefix and the first occurrence of the delimiter after the prefix
              are grouped under a single result element.

          per_page: Maximum number of objects to return per page.

          prefix: Restricts results to only those objects whose keys begin with the specified
              prefix.

          start_after: Returns objects with keys that come after the specified key in lexicographic
              order.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            page=SyncCursorPagination[ObjectListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "delimiter": delimiter,
                        "per_page": per_page,
                        "prefix": prefix,
                        "start_after": start_after,
                    },
                    object_list_params.ObjectListParams,
                ),
            ),
            model=ObjectListResponse,
        )

    def delete(
        self,
        object_key: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectDeleteResponse:
        """
        Deletes an object from an R2 bucket.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) of the object to delete. May contain slashes for path-like keys.
              Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ObjectDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ObjectDeleteResponse], ResultWrapper[ObjectDeleteResponse]),
        )

    def get(
        self,
        object_key: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        if_modified_since: str | Omit = omit,
        if_none_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """Retrieves an object from an R2 bucket.

        Returns the object body along with
        metadata headers.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) of the object to retrieve. May contain slashes for path-like
              keys. Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          if_modified_since: Returns the object only if it has been modified since the specified time. Must
              be formatted as an HTTP-date (RFC 7231), e.g. `Tue, 15 Jan 2024 10:30:00 GMT`.

          if_none_match: Returns the object only if its ETag does not match the given value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                    "If-Modified-Since": if_modified_since,
                    "If-None-Match": if_none_match,
                }
            ),
            **(extra_headers or {}),
        }
        return self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def upload(
        self,
        object_key: str,
        body: FileContent | BinaryTypes,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        cf_r2_storage_class: Literal["Standard", "InfrequentAccess"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectUploadResponse:
        """Uploads an object to an R2 bucket.

        The object body is provided as the request
        body. Returns metadata about the uploaded object.

        The maximum upload size for this endpoint is 300 MB. For most workloads, we
        recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) to assign to the object. May contain slashes for path-like keys.
              Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          body: The object body to upload.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          cf_r2_storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                    "cf-r2-storage-class": str(cf_r2_storage_class) if is_given(cf_r2_storage_class) else not_given,
                }
            ),
            **(extra_headers or {}),
        }
        extra_headers = {"Content-Type": "application/octet-stream", **(extra_headers or {})}
        return self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ObjectUploadResponse]._unwrapper,
            ),
            cast_to=cast(Type[ObjectUploadResponse], ResultWrapper[ObjectUploadResponse]),
        )


class AsyncObjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncObjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncObjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncObjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncObjectsResourceWithStreamingResponse(self)

    def list(
        self,
        bucket_name: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        delimiter: str | Omit = omit,
        per_page: int | Omit = omit,
        prefix: str | Omit = omit,
        start_after: str | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ObjectListResponse, AsyncCursorPagination[ObjectListResponse]]:
        """Lists objects in an R2 bucket.

        Returns object metadata including key, size,
        etag, last modified date, HTTP metadata, and custom metadata.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          cursor: Pagination cursor received from a previous List Objects call. Used to retrieve
              the next page of results.

          delimiter: A single character used to group keys. All keys that contain the delimiter
              between the prefix and the first occurrence of the delimiter after the prefix
              are grouped under a single result element.

          per_page: Maximum number of objects to return per page.

          prefix: Restricts results to only those objects whose keys begin with the specified
              prefix.

          start_after: Returns objects with keys that come after the specified key in lexicographic
              order.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            page=AsyncCursorPagination[ObjectListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "delimiter": delimiter,
                        "per_page": per_page,
                        "prefix": prefix,
                        "start_after": start_after,
                    },
                    object_list_params.ObjectListParams,
                ),
            ),
            model=ObjectListResponse,
        )

    async def delete(
        self,
        object_key: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectDeleteResponse:
        """
        Deletes an object from an R2 bucket.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) of the object to delete. May contain slashes for path-like keys.
              Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ObjectDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ObjectDeleteResponse], ResultWrapper[ObjectDeleteResponse]),
        )

    async def get(
        self,
        object_key: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        if_modified_since: str | Omit = omit,
        if_none_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """Retrieves an object from an R2 bucket.

        Returns the object body along with
        metadata headers.

        For most workloads, we recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) of the object to retrieve. May contain slashes for path-like
              keys. Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          if_modified_since: Returns the object only if it has been modified since the specified time. Must
              be formatted as an HTTP-date (RFC 7231), e.g. `Tue, 15 Jan 2024 10:30:00 GMT`.

          if_none_match: Returns the object only if its ETag does not match the given value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                    "If-Modified-Since": if_modified_since,
                    "If-None-Match": if_none_match,
                }
            ),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def upload(
        self,
        object_key: str,
        body: FileContent | AsyncBinaryTypes,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        cf_r2_storage_class: Literal["Standard", "InfrequentAccess"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectUploadResponse:
        """Uploads an object to an R2 bucket.

        The object body is provided as the request
        body. Returns metadata about the uploaded object.

        The maximum upload size for this endpoint is 300 MB. For most workloads, we
        recommend using R2's
        [S3-compatible API](https://developers.cloudflare.com/r2/api/s3/api/) or a
        [Worker with an R2 binding](https://developers.cloudflare.com/r2/api/workers/workers-api-reference/)
        instead.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          object_key: The key (name) to assign to the object. May contain slashes for path-like keys.
              Slashes (`/`) within the key MUST be sent literally and MUST NOT be
              percent-encoded (i.e. `%2F`); other reserved characters should be
              percent-encoded as usual.

          body: The object body to upload.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          cf_r2_storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_key:
            raise ValueError(f"Expected a non-empty value for `object_key` but received {object_key!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                    "cf-r2-storage-class": str(cf_r2_storage_class) if is_given(cf_r2_storage_class) else not_given,
                }
            ),
            **(extra_headers or {}),
        }
        extra_headers = {"Content-Type": "application/octet-stream", **(extra_headers or {})}
        return await self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/objects/{object_key}",
                account_id=account_id,
                bucket_name=bucket_name,
                object_key=object_key,
            ),
            content=await async_read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ObjectUploadResponse]._unwrapper,
            ),
            cast_to=cast(Type[ObjectUploadResponse], ResultWrapper[ObjectUploadResponse]),
        )


class ObjectsResourceWithRawResponse:
    def __init__(self, objects: ObjectsResource) -> None:
        self._objects = objects

        self.list = to_raw_response_wrapper(
            objects.list,
        )
        self.delete = to_raw_response_wrapper(
            objects.delete,
        )
        self.get = to_custom_raw_response_wrapper(
            objects.get,
            BinaryAPIResponse,
        )
        self.upload = to_raw_response_wrapper(
            objects.upload,
        )


class AsyncObjectsResourceWithRawResponse:
    def __init__(self, objects: AsyncObjectsResource) -> None:
        self._objects = objects

        self.list = async_to_raw_response_wrapper(
            objects.list,
        )
        self.delete = async_to_raw_response_wrapper(
            objects.delete,
        )
        self.get = async_to_custom_raw_response_wrapper(
            objects.get,
            AsyncBinaryAPIResponse,
        )
        self.upload = async_to_raw_response_wrapper(
            objects.upload,
        )


class ObjectsResourceWithStreamingResponse:
    def __init__(self, objects: ObjectsResource) -> None:
        self._objects = objects

        self.list = to_streamed_response_wrapper(
            objects.list,
        )
        self.delete = to_streamed_response_wrapper(
            objects.delete,
        )
        self.get = to_custom_streamed_response_wrapper(
            objects.get,
            StreamedBinaryAPIResponse,
        )
        self.upload = to_streamed_response_wrapper(
            objects.upload,
        )


class AsyncObjectsResourceWithStreamingResponse:
    def __init__(self, objects: AsyncObjectsResource) -> None:
        self._objects = objects

        self.list = async_to_streamed_response_wrapper(
            objects.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            objects.delete,
        )
        self.get = async_to_custom_streamed_response_wrapper(
            objects.get,
            AsyncStreamedBinaryAPIResponse,
        )
        self.upload = async_to_streamed_response_wrapper(
            objects.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import date
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.robots_txt.top import user_agent_directive_params
from .....types.radar.robots_txt.top.user_agent_directive_response import UserAgentDirectiveResponse

__all__ = ["UserAgentsResource", "AsyncUserAgentsResource"]


class UserAgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UserAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UserAgentsResourceWithStreamingResponse(self)

    def directive(
        self,
        *,
        date: SequenceNotStr[Union[str, date]] | Omit = omit,
        directive: Literal["ALLOW", "DISALLOW"] | Omit = omit,
        domain_category: SequenceNotStr[str] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        user_agent_category: Literal["AI"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAgentDirectiveResponse:
        """
        Retrieves the top user agents on robots.txt files.

        Args:
          date: Filters results by the specified array of dates.

          directive: Filters results by robots.txt directive.

          domain_category: Filters results by domain category.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          name: Array of names used to label the series in the response.

          user_agent_category: Filters results by user agent category.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/robots_txt/top/user_agents/directive",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "directive": directive,
                        "domain_category": domain_category,
                        "format": format,
                        "limit": limit,
                        "name": name,
                        "user_agent_category": user_agent_category,
                    },
                    user_agent_directive_params.UserAgentDirectiveParams,
                ),
                post_parser=ResultWrapper[UserAgentDirectiveResponse]._unwrapper,
            ),
            cast_to=cast(Type[UserAgentDirectiveResponse], ResultWrapper[UserAgentDirectiveResponse]),
        )


class AsyncUserAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUserAgentsResourceWithStreamingResponse(self)

    async def directive(
        self,
        *,
        date: SequenceNotStr[Union[str, date]] | Omit = omit,
        directive: Literal["ALLOW", "DISALLOW"] | Omit = omit,
        domain_category: SequenceNotStr[str] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        user_agent_category: Literal["AI"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAgentDirectiveResponse:
        """
        Retrieves the top user agents on robots.txt files.

        Args:
          date: Filters results by the specified array of dates.

          directive: Filters results by robots.txt directive.

          domain_category: Filters results by domain category.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          name: Array of names used to label the series in the response.

          user_agent_category: Filters results by user agent category.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/robots_txt/top/user_agents/directive",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date": date,
                        "directive": directive,
                        "domain_category": domain_category,
                        "format": format,
                        "limit": limit,
                        "name": name,
                        "user_agent_category": user_agent_category,
                    },
                    user_agent_directive_params.UserAgentDirectiveParams,
                ),
                post_parser=ResultWrapper[UserAgentDirectiveResponse]._unwrapper,
            ),
            cast_to=cast(Type[UserAgentDirectiveResponse], ResultWrapper[UserAgentDirectiveResponse]),
        )


class UserAgentsResourceWithRawResponse:
    def __init__(self, user_agents: UserAgentsResource) -> None:
        self._user_agents = user_agents

        self.directive = to_raw_response_wrapper(
            user_agents.directive,
        )


class AsyncUserAgentsResourceWithRawResponse:
    def __init__(self, user_agents: AsyncUserAgentsResource) -> None:
        self._user_agents = user_agents

        self.directive = async_to_raw_response_wrapper(
            user_agents.directive,
        )


class UserAgentsResourceWithStreamingResponse:
    def __init__(self, user_agents: UserAgentsResource) -> None:
        self._user_agents = user_agents

        self.directive = to_streamed_response_wrapper(
            user_agents.directive,
        )


class AsyncUserAgentsResourceWithStreamingResponse:
    def __init__(self, user_agents: AsyncUserAgentsResource) -> None:
        self._user_agents = user_agents

        self.directive = async_to_streamed_response_wrapper(
            user_agents.directive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..ssl.request_validity import RequestValidity
from ..shared.certificate_request_type import CertificateRequestType

__all__ = ["OriginCACertificate"]


class OriginCACertificate(BaseModel):
    csr: str
    """The Certificate Signing Request (CSR). Must be newline-encoded."""

    hostnames: List[str]
    """
    Array of hostnames or wildcard names bound to the certificate. Hostnames must be
    fully qualified domain names (FQDNs) belonging to zones on your account (e.g.,
    `example.com` or `sub.example.com`). Wildcards are supported only as a `*.`
    prefix for a single level (e.g., `*.example.com`). Double wildcards
    (`*.*.example.com`) and interior wildcards (`foo.*.example.com`) are not
    allowed. The wildcard suffix must be a multi-label domain (`*.example.com` is
    valid, but `*.com` is not). Unicode/IDN hostnames are accepted and automatically
    converted to punycode.
    """

    request_type: CertificateRequestType
    """
    Signature type desired on certificate ("origin-rsa" (rsa), "origin-ecc" (ecdsa),
    or "keyless-certificate" (for Keyless SSL servers).
    """

    requested_validity: RequestValidity
    """The number of days for which the certificate should be valid."""

    id: Optional[str] = None
    """Identifier."""

    certificate: Optional[str] = None
    """The Origin CA certificate. Will be newline-encoded."""

    expires_on: Optional[str] = None
    """When the certificate will expire."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Type, Optional, cast

import httpx

from ....._files import read_file_content, async_read_file_content
from ....._types import (
    Body,
    Query,
    Headers,
    NotGiven,
    BinaryTypes,
    FileContent,
    AsyncBinaryTypes,
    not_given,
)
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.dlp.dataset import Dataset
from .....types.zero_trust.dlp.datasets.new_version import NewVersion

__all__ = ["UploadResource", "AsyncUploadResource"]


class UploadResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UploadResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UploadResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UploadResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UploadResourceWithStreamingResponse(self)

    def create(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NewVersion]:
        """
        Creates a new version of a DLP dataset, allowing you to stage changes before
        activation. Used for single-column EDM and custom word lists.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/dlp/datasets/{dataset_id}/upload", account_id=account_id, dataset_id=dataset_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NewVersion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NewVersion]], ResultWrapper[NewVersion]),
        )

    def edit(
        self,
        version: int,
        dataset: FileContent | BinaryTypes,
        *,
        account_id: str,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Dataset]:
        """This is used for single-column EDMv1 and Custom Word Lists.

        The EDM format can
        only be created in the Cloudflare dashboard. For other clients, this operation
        can only be used for non-secret Custom Word Lists. The body must be a UTF-8
        encoded, newline (NL or CRNL) separated list of words to be matched.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        extra_headers = {"Content-Type": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/dlp/datasets/{dataset_id}/upload/{version}",
                account_id=account_id,
                dataset_id=dataset_id,
                version=version,
            ),
            content=read_file_content(dataset) if isinstance(dataset, os.PathLike) else dataset,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Dataset]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Dataset]], ResultWrapper[Dataset]),
        )


class AsyncUploadResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUploadResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUploadResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUploadResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUploadResourceWithStreamingResponse(self)

    async def create(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NewVersion]:
        """
        Creates a new version of a DLP dataset, allowing you to stage changes before
        activation. Used for single-column EDM and custom word lists.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/dlp/datasets/{dataset_id}/upload", account_id=account_id, dataset_id=dataset_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NewVersion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NewVersion]], ResultWrapper[NewVersion]),
        )

    async def edit(
        self,
        version: int,
        dataset: FileContent | AsyncBinaryTypes,
        *,
        account_id: str,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Dataset]:
        """This is used for single-column EDMv1 and Custom Word Lists.

        The EDM format can
        only be created in the Cloudflare dashboard. For other clients, this operation
        can only be used for non-secret Custom Word Lists. The body must be a UTF-8
        encoded, newline (NL or CRNL) separated list of words to be matched.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        extra_headers = {"Content-Type": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/dlp/datasets/{dataset_id}/upload/{version}",
                account_id=account_id,
                dataset_id=dataset_id,
                version=version,
            ),
            content=await async_read_file_content(dataset) if isinstance(dataset, os.PathLike) else dataset,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Dataset]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Dataset]], ResultWrapper[Dataset]),
        )


class UploadResourceWithRawResponse:
    def __init__(self, upload: UploadResource) -> None:
        self._upload = upload

        self.create = to_raw_response_wrapper(
            upload.create,
        )
        self.edit = to_raw_response_wrapper(
            upload.edit,
        )


class AsyncUploadResourceWithRawResponse:
    def __init__(self, upload: AsyncUploadResource) -> None:
        self._upload = upload

        self.create = async_to_raw_response_wrapper(
            upload.create,
        )
        self.edit = async_to_raw_response_wrapper(
            upload.edit,
        )


class UploadResourceWithStreamingResponse:
    def __init__(self, upload: UploadResource) -> None:
        self._upload = upload

        self.create = to_streamed_response_wrapper(
            upload.create,
        )
        self.edit = to_streamed_response_wrapper(
            upload.edit,
        )


class AsyncUploadResourceWithStreamingResponse:
    def __init__(self, upload: AsyncUploadResource) -> None:
        self._upload = upload

        self.create = async_to_streamed_response_wrapper(
            upload.create,
        )
        self.edit = async_to_streamed_response_wrapper(
            upload.edit,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AllowPolicyEditParams"]


class AllowPolicyEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    comments: Optional[str]

    is_acceptable_sender: bool
    """
    Messages from this sender will be exempted from Spam, Spoof and Bulk
    dispositions. Note - This will not exempt messages with Malicious or Suspicious
    dispositions.
    """

    is_exempt_recipient: bool
    """Messages to this recipient will bypass all detections"""

    is_recipient: bool
    """Deprecated as of July 1, 2025.

    Use `is_exempt_recipient` instead. End of life: July 1, 2026.
    """

    is_regex: bool

    is_sender: bool
    """Deprecated as of July 1, 2025.

    Use `is_trusted_sender` instead. End of life: July 1, 2026.
    """

    is_spoof: bool
    """Deprecated as of July 1, 2025.

    Use `is_acceptable_sender` instead. End of life: July 1, 2026.
    """

    is_trusted_sender: bool
    """Messages from this sender will bypass all detections and link following"""

    pattern: str

    pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"]
    """
    Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
    creating or updating policies, but may be returned for existing entries.
    """

    verify_sender: bool
    """Enforce DMARC, SPF or DKIM authentication.

    When on, Email Security only honors policies that pass authentication.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .host import Host
from .status import Status
from ..._models import BaseModel

__all__ = [
    "CertificatePackGetResponse",
    "Certificate",
    "CertificateGeoRestrictions",
    "DCVDelegationRecord",
    "ValidationError",
    "ValidationRecord",
]


class CertificateGeoRestrictions(BaseModel):
    """Specify the region where your private key can be held locally."""

    label: Optional[Literal["us", "eu", "highest_security"]] = None


class Certificate(BaseModel):
    """An individual certificate within a certificate pack."""

    id: str
    """Certificate identifier."""

    hosts: List[str]
    """Hostnames covered by this certificate."""

    status: str
    """Certificate status."""

    bundle_method: Optional[str] = None
    """Certificate bundle method."""

    expires_on: Optional[datetime] = None
    """When the certificate from the authority expires."""

    geo_restrictions: Optional[CertificateGeoRestrictions] = None
    """Specify the region where your private key can be held locally."""

    issuer: Optional[str] = None
    """The certificate authority that issued the certificate."""

    modified_on: Optional[datetime] = None
    """When the certificate was last modified."""

    priority: Optional[float] = None
    """The order/priority in which the certificate will be used."""

    signature: Optional[str] = None
    """The type of hash used for the certificate."""

    uploaded_on: Optional[datetime] = None
    """When the certificate was uploaded to Cloudflare."""

    zone_id: Optional[str] = None
    """Identifier."""


class DCVDelegationRecord(BaseModel):
    """Certificate's required validation record."""

    cname: Optional[str] = None
    """The CNAME record hostname for DCV delegation."""

    cname_target: Optional[str] = None
    """The CNAME record target value for DCV delegation."""

    emails: Optional[List[str]] = None
    """
    The set of email addresses that the certificate authority (CA) will use to
    complete domain validation.
    """

    http_body: Optional[str] = None
    """
    The content that the certificate authority (CA) will expect to find at the
    http_url during the domain validation.
    """

    http_url: Optional[str] = None
    """The url that will be checked during domain validation."""

    status: Optional[str] = None
    """Status of the validation record."""

    txt_name: Optional[str] = None
    """
    The hostname that the certificate authority (CA) will check for a TXT record
    during domain validation .
    """

    txt_value: Optional[str] = None
    """
    The TXT record that the certificate authority (CA) will check during domain
    validation.
    """


class ValidationError(BaseModel):
    message: Optional[str] = None
    """A domain validation error."""


class ValidationRecord(BaseModel):
    """Certificate's required validation record."""

    cname: Optional[str] = None
    """The CNAME record hostname for DCV delegation."""

    cname_target: Optional[str] = None
    """The CNAME record target value for DCV delegation."""

    emails: Optional[List[str]] = None
    """
    The set of email addresses that the certificate authority (CA) will use to
    complete domain validation.
    """

    http_body: Optional[str] = None
    """
    The content that the certificate authority (CA) will expect to find at the
    http_url during the domain validation.
    """

    http_url: Optional[str] = None
    """The url that will be checked during domain validation."""

    status: Optional[str] = None
    """Status of the validation record."""

    txt_name: Optional[str] = None
    """
    The hostname that the certificate authority (CA) will check for a TXT record
    during domain validation .
    """

    txt_value: Optional[str] = None
    """
    The TXT record that the certificate authority (CA) will check during domain
    validation.
    """


class CertificatePackGetResponse(BaseModel):
    """A certificate pack with all its properties."""

    id: str
    """Identifier."""

    certificates: List[Certificate]
    """Array of certificates in this pack."""

    hosts: List[Host]
    """Comma separated list of valid host names for the certificate packs.

    Must contain the zone apex, may not contain more than 50 hosts, and may not be
    empty.
    """

    status: Status
    """Status of certificate pack."""

    type: Literal[
        "mh_custom", "managed_hostname", "sni_custom", "universal", "advanced", "total_tls", "keyless", "legacy_custom"
    ]
    """Type of certificate pack."""

    certificate_authority: Optional[Literal["google", "lets_encrypt", "ssl_com"]] = None
    """Certificate Authority selected for the order.

    For information on any certificate authority specific details or restrictions
    [see this page for more details.](https://developers.cloudflare.com/ssl/reference/certificate-authorities)
    """

    cloudflare_branding: Optional[bool] = None
    """Whether or not to add Cloudflare Branding for the order.

    This will add a subdomain of sni.cloudflaressl.com as the Common Name if set to
    true.
    """

    dcv_delegation_records: Optional[List[DCVDelegationRecord]] = None
    """DCV Delegation records for domain validation."""

    primary_certificate: Optional[str] = None
    """Identifier of the primary certificate in a pack."""

    validation_errors: Optional[List[ValidationError]] = None
    """
    Domain validation errors that have been received by the certificate authority
    (CA).
    """

    validation_method: Optional[Literal["txt", "http", "email"]] = None
    """Validation Method selected for the order."""

    validation_records: Optional[List[ValidationRecord]] = None
    """Certificates' validation records."""

    validity_days: Optional[Literal[14, 30, 90, 365]] = None
    """Validity Days selected for the order."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.pipelines import sink_list_params, sink_create_params, sink_delete_params
from ...types.pipelines.sink_get_response import SinkGetResponse
from ...types.pipelines.sink_list_response import SinkListResponse
from ...types.pipelines.sink_create_response import SinkCreateResponse

__all__ = ["SinksResource", "AsyncSinksResource"]


class SinksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SinksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        type: Literal["r2", "r2_data_catalog"],
        config: sink_create_params.Config | Omit = omit,
        format: sink_create_params.Format | Omit = omit,
        schema: sink_create_params.Schema | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SinkCreateResponse:
        """
        Create a new Sink.

        Args:
          account_id: Specifies the public ID of the account.

          name: Defines the name of the Sink.

          type: Specifies the type of sink.

          config: Defines the configuration of the R2 Sink.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pipelines/v1/sinks", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "config": config,
                    "format": format,
                    "schema": schema,
                },
                sink_create_params.SinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SinkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SinkCreateResponse], ResultWrapper[SinkCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        pipeline_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[SinkListResponse]:
        """
        List/Filter Sinks in Account.

        Args:
          account_id: Specifies the public ID of the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pipelines/v1/sinks", account_id=account_id),
            page=SyncV4PagePaginationArray[SinkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "pipeline_id": pipeline_id,
                    },
                    sink_list_params.SinkListParams,
                ),
            ),
            model=SinkListResponse,
        )

    def delete(
        self,
        sink_id: str,
        *,
        account_id: str,
        force: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Pipeline in Account.

        Args:
          account_id: Specifies the public ID of the account.

          sink_id: Specifies the publid ID of the sink.

          force: Delete sink forcefully, including deleting any dependent pipelines.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sink_id:
            raise ValueError(f"Expected a non-empty value for `sink_id` but received {sink_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/v1/sinks/{sink_id}", account_id=account_id, sink_id=sink_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"force": force}, sink_delete_params.SinkDeleteParams),
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        sink_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SinkGetResponse:
        """
        Get Sink Details.

        Args:
          account_id: Specifies the public ID of the account.

          sink_id: Specifies the publid ID of the sink.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sink_id:
            raise ValueError(f"Expected a non-empty value for `sink_id` but received {sink_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/pipelines/v1/sinks/{sink_id}", account_id=account_id, sink_id=sink_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SinkGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SinkGetResponse], ResultWrapper[SinkGetResponse]),
        )


class AsyncSinksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSinksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        type: Literal["r2", "r2_data_catalog"],
        config: sink_create_params.Config | Omit = omit,
        format: sink_create_params.Format | Omit = omit,
        schema: sink_create_params.Schema | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SinkCreateResponse:
        """
        Create a new Sink.

        Args:
          account_id: Specifies the public ID of the account.

          name: Defines the name of the Sink.

          type: Specifies the type of sink.

          config: Defines the configuration of the R2 Sink.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pipelines/v1/sinks", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "config": config,
                    "format": format,
                    "schema": schema,
                },
                sink_create_params.SinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SinkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SinkCreateResponse], ResultWrapper[SinkCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        pipeline_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SinkListResponse, AsyncV4PagePaginationArray[SinkListResponse]]:
        """
        List/Filter Sinks in Account.

        Args:
          account_id: Specifies the public ID of the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pipelines/v1/sinks", account_id=account_id),
            page=AsyncV4PagePaginationArray[SinkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "pipeline_id": pipeline_id,
                    },
                    sink_list_params.SinkListParams,
                ),
            ),
            model=SinkListResponse,
        )

    async def delete(
        self,
        sink_id: str,
        *,
        account_id: str,
        force: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Pipeline in Account.

        Args:
          account_id: Specifies the public ID of the account.

          sink_id: Specifies the publid ID of the sink.

          force: Delete sink forcefully, including deleting any dependent pipelines.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sink_id:
            raise ValueError(f"Expected a non-empty value for `sink_id` but received {sink_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/v1/sinks/{sink_id}", account_id=account_id, sink_id=sink_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"force": force}, sink_delete_params.SinkDeleteParams),
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        sink_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SinkGetResponse:
        """
        Get Sink Details.

        Args:
          account_id: Specifies the public ID of the account.

          sink_id: Specifies the publid ID of the sink.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sink_id:
            raise ValueError(f"Expected a non-empty value for `sink_id` but received {sink_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/pipelines/v1/sinks/{sink_id}", account_id=account_id, sink_id=sink_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SinkGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SinkGetResponse], ResultWrapper[SinkGetResponse]),
        )


class SinksResourceWithRawResponse:
    def __init__(self, sinks: SinksResource) -> None:
        self._sinks = sinks

        self.create = to_raw_response_wrapper(
            sinks.create,
        )
        self.list = to_raw_response_wrapper(
            sinks.list,
        )
        self.delete = to_raw_response_wrapper(
            sinks.delete,
        )
        self.get = to_raw_response_wrapper(
            sinks.get,
        )


class AsyncSinksResourceWithRawResponse:
    def __init__(self, sinks: AsyncSinksResource) -> None:
        self._sinks = sinks

        self.create = async_to_raw_response_wrapper(
            sinks.create,
        )
        self.list = async_to_raw_response_wrapper(
            sinks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sinks.delete,
        )
        self.get = async_to_raw_response_wrapper(
            sinks.get,
        )


class SinksResourceWithStreamingResponse:
    def __init__(self, sinks: SinksResource) -> None:
        self._sinks = sinks

        self.create = to_streamed_response_wrapper(
            sinks.create,
        )
        self.list = to_streamed_response_wrapper(
            sinks.list,
        )
        self.delete = to_streamed_response_wrapper(
            sinks.delete,
        )
        self.get = to_streamed_response_wrapper(
            sinks.get,
        )


class AsyncSinksResourceWithStreamingResponse:
    def __init__(self, sinks: AsyncSinksResource) -> None:
        self._sinks = sinks

        self.create = async_to_streamed_response_wrapper(
            sinks.create,
        )
        self.list = async_to_streamed_response_wrapper(
            sinks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sinks.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            sinks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.realtime_kit import preset_get_params, preset_create_params, preset_update_params
from ...types.realtime_kit.preset_get_response import PresetGetResponse
from ...types.realtime_kit.preset_create_response import PresetCreateResponse
from ...types.realtime_kit.preset_delete_response import PresetDeleteResponse
from ...types.realtime_kit.preset_update_response import PresetUpdateResponse
from ...types.realtime_kit.preset_get_preset_by_id_response import PresetGetPresetByIDResponse

__all__ = ["PresetsResource", "AsyncPresetsResource"]


class PresetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PresetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PresetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PresetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PresetsResourceWithStreamingResponse(self)

    def create(
        self,
        app_id: str,
        *,
        account_id: str,
        config: preset_create_params.Config,
        name: str,
        ui: preset_create_params.UI,
        permissions: preset_create_params.Permissions | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetCreateResponse:
        """
        Creates a preset belonging to the current App

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: Name of the preset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/realtime/kit/{app_id}/presets", account_id=account_id, app_id=app_id),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "ui": ui,
                    "permissions": permissions,
                },
                preset_create_params.PresetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetCreateResponse,
        )

    def update(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        config: preset_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        permissions: preset_update_params.Permissions | Omit = omit,
        ui: preset_update_params.UI | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetUpdateResponse:
        """
        Update a preset by the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: Name of the preset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "permissions": permissions,
                    "ui": ui,
                },
                preset_update_params.PresetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetUpdateResponse,
        )

    def delete(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetDeleteResponse:
        """
        Deletes a preset using the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetDeleteResponse,
        )

    def get(
        self,
        app_id: str,
        *,
        account_id: str,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetGetResponse:
        """
        Fetches all the presets belonging to an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/realtime/kit/{app_id}/presets", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_no": page_no,
                        "per_page": per_page,
                    },
                    preset_get_params.PresetGetParams,
                ),
            ),
            cast_to=PresetGetResponse,
        )

    def get_preset_by_id(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetGetPresetByIDResponse:
        """
        Fetches details of a preset using the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetGetPresetByIDResponse,
        )


class AsyncPresetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPresetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPresetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPresetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPresetsResourceWithStreamingResponse(self)

    async def create(
        self,
        app_id: str,
        *,
        account_id: str,
        config: preset_create_params.Config,
        name: str,
        ui: preset_create_params.UI,
        permissions: preset_create_params.Permissions | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetCreateResponse:
        """
        Creates a preset belonging to the current App

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: Name of the preset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/realtime/kit/{app_id}/presets", account_id=account_id, app_id=app_id),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "ui": ui,
                    "permissions": permissions,
                },
                preset_create_params.PresetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetCreateResponse,
        )

    async def update(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        config: preset_update_params.Config | Omit = omit,
        name: str | Omit = omit,
        permissions: preset_update_params.Permissions | Omit = omit,
        ui: preset_update_params.UI | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetUpdateResponse:
        """
        Update a preset by the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: Name of the preset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "permissions": permissions,
                    "ui": ui,
                },
                preset_update_params.PresetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetUpdateResponse,
        )

    async def delete(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetDeleteResponse:
        """
        Deletes a preset using the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetDeleteResponse,
        )

    async def get(
        self,
        app_id: str,
        *,
        account_id: str,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetGetResponse:
        """
        Fetches all the presets belonging to an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/realtime/kit/{app_id}/presets", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_no": page_no,
                        "per_page": per_page,
                    },
                    preset_get_params.PresetGetParams,
                ),
            ),
            cast_to=PresetGetResponse,
        )

    async def get_preset_by_id(
        self,
        preset_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PresetGetPresetByIDResponse:
        """
        Fetches details of a preset using the provided preset ID

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not preset_id:
            raise ValueError(f"Expected a non-empty value for `preset_id` but received {preset_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/presets/{preset_id}",
                account_id=account_id,
                app_id=app_id,
                preset_id=preset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PresetGetPresetByIDResponse,
        )


class PresetsResourceWithRawResponse:
    def __init__(self, presets: PresetsResource) -> None:
        self._presets = presets

        self.create = to_raw_response_wrapper(
            presets.create,
        )
        self.update = to_raw_response_wrapper(
            presets.update,
        )
        self.delete = to_raw_response_wrapper(
            presets.delete,
        )
        self.get = to_raw_response_wrapper(
            presets.get,
        )
        self.get_preset_by_id = to_raw_response_wrapper(
            presets.get_preset_by_id,
        )


class AsyncPresetsResourceWithRawResponse:
    def __init__(self, presets: AsyncPresetsResource) -> None:
        self._presets = presets

        self.create = async_to_raw_response_wrapper(
            presets.create,
        )
        self.update = async_to_raw_response_wrapper(
            presets.update,
        )
        self.delete = async_to_raw_response_wrapper(
            presets.delete,
        )
        self.get = async_to_raw_response_wrapper(
            presets.get,
        )
        self.get_preset_by_id = async_to_raw_response_wrapper(
            presets.get_preset_by_id,
        )


class PresetsResourceWithStreamingResponse:
    def __init__(self, presets: PresetsResource) -> None:
        self._presets = presets

        self.create = to_streamed_response_wrapper(
            presets.create,
        )
        self.update = to_streamed_response_wrapper(
            presets.update,
        )
        self.delete = to_streamed_response_wrapper(
            presets.delete,
        )
        self.get = to_streamed_response_wrapper(
            presets.get,
        )
        self.get_preset_by_id = to_streamed_response_wrapper(
            presets.get_preset_by_id,
        )


class AsyncPresetsResourceWithStreamingResponse:
    def __init__(self, presets: AsyncPresetsResource) -> None:
        self._presets = presets

        self.create = async_to_streamed_response_wrapper(
            presets.create,
        )
        self.update = async_to_streamed_response_wrapper(
            presets.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            presets.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            presets.get,
        )
        self.get_preset_by_id = async_to_streamed_response_wrapper(
            presets.get_preset_by_id,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PermissionGroupListResponse", "Meta"]


class Meta(BaseModel):
    """Attributes associated to the permission group."""

    key: Optional[str] = None

    value: Optional[str] = None


class PermissionGroupListResponse(BaseModel):
    """
    A named group of permissions that map to a group of operations against resources.
    """

    id: str
    """Identifier of the permission group."""

    meta: Optional[Meta] = None
    """Attributes associated to the permission group."""

    name: Optional[str] = None
    """Name of the permission group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.brand_protection import match_get_params, match_download_params
from ...types.brand_protection.match_get_response import MatchGetResponse
from ...types.brand_protection.match_download_response import MatchDownloadResponse

__all__ = ["MatchesResource", "AsyncMatchesResource"]


class MatchesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MatchesResourceWithStreamingResponse(self)

    def download(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        include_domain_id: bool | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MatchDownloadResponse:
        """
        Return matches as CSV for string queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/brand-protection/matches/download", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "include_domain_id": include_domain_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    match_download_params.MatchDownloadParams,
                ),
            ),
            cast_to=MatchDownloadResponse,
        )

    def get(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        include_domain_id: bool | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MatchGetResponse:
        """
        Return matches for string queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/brand-protection/matches", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "include_domain_id": include_domain_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    match_get_params.MatchGetParams,
                ),
            ),
            cast_to=MatchGetResponse,
        )


class AsyncMatchesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMatchesResourceWithStreamingResponse(self)

    async def download(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        include_domain_id: bool | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MatchDownloadResponse:
        """
        Return matches as CSV for string queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/brand-protection/matches/download", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "include_domain_id": include_domain_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    match_download_params.MatchDownloadParams,
                ),
            ),
            cast_to=MatchDownloadResponse,
        )

    async def get(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        include_domain_id: bool | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MatchGetResponse:
        """
        Return matches for string queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/brand-protection/matches", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "include_domain_id": include_domain_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    match_get_params.MatchGetParams,
                ),
            ),
            cast_to=MatchGetResponse,
        )


class MatchesResourceWithRawResponse:
    def __init__(self, matches: MatchesResource) -> None:
        self._matches = matches

        self.download = to_raw_response_wrapper(
            matches.download,
        )
        self.get = to_raw_response_wrapper(
            matches.get,
        )


class AsyncMatchesResourceWithRawResponse:
    def __init__(self, matches: AsyncMatchesResource) -> None:
        self._matches = matches

        self.download = async_to_raw_response_wrapper(
            matches.download,
        )
        self.get = async_to_raw_response_wrapper(
            matches.get,
        )


class MatchesResourceWithStreamingResponse:
    def __init__(self, matches: MatchesResource) -> None:
        self._matches = matches

        self.download = to_streamed_response_wrapper(
            matches.download,
        )
        self.get = to_streamed_response_wrapper(
            matches.get,
        )


class AsyncMatchesResourceWithStreamingResponse:
    def __init__(self, matches: AsyncMatchesResource) -> None:
        self._matches = matches

        self.download = async_to_streamed_response_wrapper(
            matches.download,
        )
        self.get = async_to_streamed_response_wrapper(
            matches.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["FipsSettings"]


class FipsSettings(BaseModel):
    """Specify FIPS settings."""

    tls: Optional[bool] = None
    """Enforce cipher suites and TLS versions compliant with FIPS 140-2."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OutputUpdateParams"]


class OutputUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    live_input_identifier: Required[str]
    """A unique identifier for a live input."""

    enabled: Required[bool]
    """
    When enabled, live video streamed to the associated live input will be sent to
    the output URL. When disabled, live video will not be sent to the output URL,
    even when streaming to the associated live input. Use this to control precisely
    when you start and stop simulcasting to specific destinations like YouTube and
    Twitch.
    """
